@@ -253,7 +253,7 @@ require_dbus_session (int      argc,
                               TRUE);
 
         /* +2 for our new arguments, +1 for NULL */
-        new_argv = g_malloc (argc + 3 * sizeof (*argv));
+        new_argv = g_malloc ((argc + 3) * sizeof (*argv));
 
         new_argv[0] = "dbus-launch";
         new_argv[1] = "--exit-with-session";
@@ -296,6 +296,7 @@ main (int argc, char **argv)
                 { "whale", 0, 0, G_OPTION_ARG_NONE, &please_fail, N_("Show the fail whale dialog for testing"), NULL },
                 { NULL, 0, 0, 0, NULL, NULL, NULL }
         };
+        char *qt_platform_theme_new = NULL;
 
         /* Make sure that we have a session bus */
         if (!require_dbus_session (argc, argv, &error)) {
@@ -306,11 +307,13 @@ main (int argc, char **argv)
         bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
         textdomain (GETTEXT_PACKAGE);
 
-        GSettings *settings = g_settings_new ("org.cinnamon.SessionManager");
+        GSettings *settings;
+
+        settings = g_settings_new ("org.cinnamon.SessionManager");
         if (g_settings_get_boolean (settings, "debug")) {
             debug = TRUE;
         }
-        g_object_unref(settings);
+        g_clear_object (&settings);
 
         sa.sa_handler = SIG_IGN;
         sa.sa_flags = 0;
@@ -381,8 +384,34 @@ main (int argc, char **argv)
          */
         csm_util_setenv ("GNOME_DESKTOP_SESSION_ID", "this-is-deprecated");
 
-        /* Make QT5 apps follow the GTK style */
-        csm_util_setenv ("QT_STYLE_OVERRIDE", "gtk");
+
+        /* Make QT5 apps follow the GTK style. Starting with QT 5.7, a different
+         * env var has to be set than what worked in previous versions.
+         */
+        qt_platform_theme_new = HAVE_QT57 ? "qt5ct" : "qgnomeplatform";
+
+        if (NULL == g_getenv ("QT_QPA_PLATFORMTHEME")) {
+            csm_util_setenv ("QT_QPA_PLATFORMTHEME", qt_platform_theme_new);
+        }
+
+        if ( ! HAVE_QT57 && NULL == g_getenv ("QT_STYLE_OVERRIDE") ) {
+            csm_util_setenv ("QT_STYLE_OVERRIDE", "gtk");
+
+        } else if (HAVE_QT57 && NULL != g_getenv ("QT_STYLE_OVERRIDE")) {
+            g_unsetenv ("QT_STYLE_OVERRIDE");
+        }
+
+
+        /* GTK Overlay scrollbars */
+        settings = g_settings_new ("org.cinnamon.desktop.interface");
+
+        if (g_settings_get_boolean (settings, "gtk-overlay-scrollbars")) {
+            csm_util_setenv ("GTK_OVERLAY_SCROLLING", "1");
+        } else {
+            csm_util_setenv ("GTK_OVERLAY_SCROLLING", "0");
+        }
+
+        g_clear_object (&settings);
 
         client_store = csm_store_new ();
 
